/**
 * Converts any string to camel-case format.
 * @param   {String}    str           String to convert.
 * @return  {String}                  The formatted string.
 */
export var toCamelCase = function toCamelCase(str) {
  // remove spaces, dashes and underscores from the begining of the string
  // /^[A-Z]+$/ -> lowercases the string if it's all uppercase
  var cleanString = str.replace(/[^\w\s_-]/g, '').replace(/^[A-Z]+$/, function (w) {
    return w.toLowerCase();
  }); // if it's in snakecase, convert it to camelcase

  if (/(.*?)[\s_-]/.test(cleanString)) {
    var parts = cleanString.replace(/[\s_-]|[a-z](?=[A-Z])/g, function (w) {
      return /[\s_-]/.test(w) ? ':' : w + ":";
    }).split(':');
    var transformedParts = parts.map(function (w, i) {
      return i === 0 ? w.toLowerCase() : "" + w[0].toUpperCase() + w.slice(1).toLowerCase();
    });
    return transformedParts.join('');
  } // if it's already in camelcase, return it


  if (/([a-z][A-Z])+/.test(cleanString)) {
    return cleanString;
  }

  return cleanString;
};
/**
 * Converts any string to snake-case format.
 * @param   {String}    str           String to convert.
 * @return  {String}                  The formatted string.
 */

export var toSnakeCase = function toSnakeCase(str) {
  var camelCase = toCamelCase(str);
  return camelCase.replace(/[a-z](?=[A-Z])/g, function (w) {
    return w[0] + "_";
  }).toLowerCase();
};